﻿Imports Ivi.Visa.Interop
Imports System.Windows.Forms
Imports System.Threading.Tasks
Imports System.Text
Imports System.Linq
Imports System.Drawing
Imports System.Data
Imports System.ComponentModel
Imports System.Collections.Generic
Imports System


Public Class Form1

    Private Const IF_TMC As String = "USB-TMC"
    Private Const IF_RSCDC As String = "RS-232C/USB-CDC"
    Private Const IF_LAN As String = "LAN Socket"
    Private Const IF_GPIB As String = "GP-IB"
    Private Const IF_LXI As String = "LXI"

    Private usb As IUsb
    Private seri As ISerial
    Private tcpsoket As ITcpipSocket
    Private gpib As IGpib
    Private tcplxi As ITcpipInstr

    Private VISA_RM As IResourceManager = New Ivi.Visa.Interop.ResourceManager()
    Private VISA_CMB As IMessage
    Private Sub cmbIFType_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cmbIFType.SelectedIndexChanged
        cmbDeviceList.Items.Clear()
        btnOpenClose.Enabled = False
        cmbDeviceList.Text = ""

        Select Case cmbIFType.Text
            Case IF_TMC
                txtComProperty.Visible = False
                lblComPropertyTitle.Visible = False
                txtComProperty.Text = "9600"
                Exit Select

            Case IF_RSCDC
                txtComProperty.Visible = True
                lblComPropertyTitle.Visible = True
                lblComPropertyTitle.Text = "RS-232C Baudrate"
                txtComProperty.Text = "9600"
                Exit Select

            Case IF_LAN
                txtComProperty.Visible = True
                lblComPropertyTitle.Visible = True
                lblComPropertyTitle.Text = "TCPIP ADDR::PORT"
                txtComProperty.Text = "192.168.1.1::5025"
                btnOpenClose.Enabled = True
                Exit Select

            Case IF_GPIB
                txtComProperty.Visible = False
                lblComPropertyTitle.Visible = False
                lblComPropertyTitle.Text = "GP-IB Address"
                txtComProperty.Text = ""
                Exit Select

            Case IF_LXI
                txtComProperty.Visible = True
                lblComPropertyTitle.Visible = True
                lblComPropertyTitle.Text = "TCPIP ADDR"
                txtComProperty.Text = ""
                Exit Select
            Case Else

                Exit Select
        End Select

    End Sub

    Private Sub btnFindRc_Click(sender As Object, e As EventArgs) Handles btnFindRc.Click
        ' FindResourceはNI-MAXに登録されているリソースを検索します。未登録は表示されません。
        Try
            Dim VISAResources As String() = Nothing
            System.Windows.Forms.Cursor.Current = Cursors.WaitCursor
            Select Case cmbIFType.Text
                Case IF_GPIB
                    VISAResources = VISA_RM.FindRsrc("GPIB?*INSTR")
                    Exit Select

                Case IF_TMC
                    VISAResources = VISA_RM.FindRsrc("USB?*INSTR")
                    Exit Select

                Case IF_RSCDC
                    VISAResources = VISA_RM.FindRsrc("ASRL?*")
                    Exit Select

                Case IF_LAN
                    VISAResources = VISA_RM.FindRsrc("TCPIP0::" + txtComProperty.Text.Trim() + "::Socket")
                    Exit Select

                Case IF_LXI
                    VISAResources = VISA_RM.FindRsrc("TCPIP0::" + txtComProperty.Text.Trim() + "::inst0::INSTR")
                    Exit Select
            End Select

            cmbDeviceList.Items.Clear()
            cmbDeviceList.Text = ""
            For Each s As String In VISAResources
                cmbDeviceList.Items.Add(s)
            Next

            If cmbDeviceList.Items.Count > 0 Then
                cmbDeviceList.SelectedIndex = 0
                btnOpenClose.Enabled = True
            End If
        Catch ex As Exception
            MessageBox.Show("VISA Device is not found!!", "ERROR", MessageBoxButtons.OK)
        Finally
            System.Windows.Forms.Cursor.Current = Cursors.[Default]
        End Try

    End Sub


    Private Sub btnOpenClose_Click(sender As Object, e As EventArgs) Handles btnOpenClose.Click
        Try
            System.Windows.Forms.Cursor.Current = Cursors.WaitCursor
            If btnOpenClose.Text = "OPEN" Then
                btnOpenClose.Text = "CLOSE"
                btnFindRc.Enabled = False
                cmbIFType.Enabled = False
                cmbDeviceList.Enabled = False

                VISA_CMB = CType(VISA_RM.Open(cmbDeviceList.Text.Trim(), AccessMode.NO_LOCK, 0, ""), IMessage)
                VISA_CMB.Timeout = 1000
                VISA_CMB.TerminationCharacter = 10
                VISA_CMB.TerminationCharacterEnabled = False
                VISA_CMB.SendEndEnabled = True
                txtComProperty.Enabled = False
                btnReceive.Enabled = True
                btnSend.Enabled = True

                Select Case cmbIFType.Text
                    Case IF_TMC
                        ' USB-TMC
                        usb = CType(VISA_CMB, IUsb)
                        Exit Select

                    Case IF_RSCDC
                        seri = CType(VISA_CMB, ISerial)
                        '文字列を整数へ変換
                        seri.BaudRate = Integer.Parse(txtComProperty.Text)
                        seri.DataBits = 8
                        seri.StopBits = SerialStopBits.ASRL_STOP_ONE
                        seri.Parity = SerialParity.ASRL_PAR_NONE
                        seri.FlowControl = SerialFlowControl.ASRL_FLOW_NONE
                        Exit Select

                    Case IF_LAN
                        tcpsoket = CType(VISA_CMB, ITcpipSocket)
                        Exit Select

                    Case IF_GPIB
                        gpib = CType(VISA_CMB, IGpib)
                        Exit Select

                    Case IF_LXI
                        tcplxi = CType(VISA_CMB, ITcpipInstr)
                        Exit Select
                    Case Else

                        Exit Select
                End Select
                VISA_CMB.TerminationCharacterEnabled = True
            Else
                btnOpenClose.Text = "OPEN"
                btnFindRc.Enabled = True
                cmbIFType.Enabled = True
                cmbDeviceList.Enabled = True
                txtComProperty.Enabled = True
                btnReceive.Enabled = False
                btnSend.Enabled = False
                lblReceiveMessage.Text = ""
                VISA_CMB.Close()
            End If
        Catch ex As Exception
            MessageBox.Show("VISA OPEN ERROR!!", "ERROR", MessageBoxButtons.OK)
        Finally
            System.Windows.Forms.Cursor.Current = Cursors.[Default]
        End Try

    End Sub


    Private Sub btnSend_Click(sender As Object, e As EventArgs) Handles btnSend.Click
        Try
            Dim textToWrite As String = ReplaceCommonEscapeSequences(txtSendMessage.Text) + vbLf
            VISA_CMB.WriteString(textToWrite)

            If textToWrite.IndexOf("?") >= 0 Then
                '文字列に？が含まれている場合
                System.Threading.Thread.Sleep(200)
                Dim responseString As String = VISA_CMB.ReadString(256)
                lblReceiveMessage.Text = InsertCommonEscapeSequences(responseString)
            End If
        Catch exp As Exception
            MessageBox.Show(exp.Message)
        End Try

    End Sub

    Private Sub btnReceive_Click(sender As Object, e As EventArgs) Handles btnReceive.Click
        Cursor.Current = Cursors.WaitCursor
        Try
            Dim responseString As String = VISA_CMB.ReadString(256)
            lblReceiveMessage.Text = InsertCommonEscapeSequences(responseString)
        Catch exp As Exception
            MessageBox.Show(exp.Message)
        Finally
            Cursor.Current = Cursors.[Default]
        End Try

    End Sub
    Private Function ReplaceCommonEscapeSequences(s As String) As String
        '\を\\に変換
        Return s.Replace("\n", vbLf).Replace("\r", vbCr)
    End Function
    Private Function InsertCommonEscapeSequences(s As String) As String
        '\\を\に変換
        Return s.Replace(vbLf, "\n").Replace(vbCr, "\r")
    End Function

    Private Sub form_closing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing
        If VISA_CMB IsNot Nothing Then
            VISA_CMB.Close()
            VISA_CMB = Nothing
        End If
    End Sub

    Private Sub form_load(sender As Object, e As EventArgs) Handles MyBase.Load
        System.Threading.Thread.CurrentThread.CurrentCulture = System.Globalization.CultureInfo.InvariantCulture

        cmbIFType.Items.Add(IF_TMC)
        cmbIFType.Items.Add(IF_RSCDC)
        cmbIFType.Items.Add(IF_LAN)
        cmbIFType.Items.Add(IF_GPIB)
        cmbIFType.Items.Add(IF_LXI)

        cmbIFType.SelectedIndex = 0
        txtSendMessage.Text = "*IDN?"
        lblReceiveMessage.Text = [String].Empty
        btnOpenClose.Text = "OPEN"
        btnOpenClose.Enabled = False
        btnFindRc.Enabled = True
        cmbIFType.Enabled = True
        cmbDeviceList.Enabled = True
        txtComProperty.Enabled = True
        btnReceive.Enabled = False
        btnSend.Enabled = False
    End Sub
End Class
